import os
import unittest
from datetime import datetime

from maldump.av_manager import AVManager


class TestExport(unittest.TestCase):

    # Prepare environment
    @classmethod
    def setUpClass(cls):
        os.chdir('test/root')
        cls.avs = [av.export() for av in AVManager.avs]

    def test_list_not_empty(self):
        self.assertIsNotNone(self.avs)

    def test_timestamp(self):
        for av in self.avs:
            for entry in av:
                self.assertIsInstance(entry.timestamp, datetime)

    def test_path_contains_eicar(self):
        for av in self.avs:
            for entry in av:
                self.assertIsNotNone(entry.path)
                self.assertIn('eicar', entry.path)

    def test_file_size(self):
        for av in self.avs:
            for entry in av:
                self.assertEqual(68, entry.size)

    def test_md5_hash(self):
        for av in self.avs:
            for entry in av:
                self.assertEqual(entry.md5, '44d88612fea8a8f36de82e1278abb02f')

    def test_file_is_eicar(self):
        for av in self.avs:
            for entry in av:
                self.assertIsInstance(entry.malfile, bytes)
                self.assertEqual(
                    entry.malfile,
                    br'X5O!P%@AP[4\PZX54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*'
                )


if __name__ == '__main__':
    unittest.main()



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import maldump.utils as utils


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class WindefResourceData(KaitaiStruct):
    """Creator: Florian Bausch, ERNW Research GmbH, https://ernw-research.de
    License: CC-BY-SA-4.0 https://creativecommons.org/licenses/by-sa/4.0/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self._raw__raw_encryptedfile = self._io.read_bytes_full()
        _process = utils.CustomArc4(b"\x1E\x87\x78\x1B\x8D\xBA\xA8\x44\xCE\x69\x70\x2C\x0C\x78\xB7\x86\xA3\xF6\x23\xB7\x38\xF5\xED\xF9\xAF\x83\x53\x0F\xB3\xFC\x54\xFA\xA2\x1E\xB9\xCF\x13\x31\xFD\x0F\x0D\xA9\x54\xF6\x87\xCB\x9E\x18\x27\x96\x97\x90\x0E\x53\xFB\x31\x7C\x9C\xBC\xE4\x8E\x23\xD0\x53\x71\xEC\xC1\x59\x51\xB8\xF3\x64\x9D\x7C\xA3\x3E\xD6\x8D\xC9\x04\x7E\x82\xC9\xBA\xAD\x97\x99\xD0\xD4\x58\xCB\x84\x7C\xA9\xFF\xBE\x3C\x8A\x77\x52\x33\x55\x7D\xDE\x13\xA8\xB1\x40\x87\xCC\x1B\xC8\xF1\x0F\x6E\xCD\xD0\x83\xA9\x59\xCF\xF8\x4A\x9D\x1D\x50\x75\x5E\x3E\x19\x18\x18\xAF\x23\xE2\x29\x35\x58\x76\x6D\x2C\x07\xE2\x57\x12\xB2\xCA\x0B\x53\x5E\xD8\xF6\xC5\x6C\xE7\x3D\x24\xBD\xD0\x29\x17\x71\x86\x1A\x54\xB4\xC2\x85\xA9\xA3\xDB\x7A\xCA\x6D\x22\x4A\xEA\xCD\x62\x1D\xB9\xF2\xA2\x2E\xD1\xE9\xE1\x1D\x75\xBE\xD7\xDC\x0E\xCB\x0A\x8E\x68\xA2\xFF\x12\x63\x40\x8D\xC8\x08\xDF\xFD\x16\x4B\x11\x67\x74\xCD\x0B\x9B\x8D\x05\x41\x1E\xD6\x26\x2E\x42\x9B\xA4\x95\x67\x6B\x83\x98\xDB\x2F\x35\xD3\xC1\xB9\xCE\xD5\x26\x36\xF2\x76\x5E\x1A\x95\xCB\x7C\xA4\xC3\xDD\xAB\xDD\xBF\xF3\x82\x53")
        self._raw_encryptedfile = _process.decode(self._raw__raw_encryptedfile)
        _io__raw_encryptedfile = KaitaiStream(BytesIO(self._raw_encryptedfile))
        self.encryptedfile = WindefResourceData.Rc4encrypted(_io__raw_encryptedfile, self, self._root)

    class Rc4encrypted(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.fixed = self._io.read_bytes(8)
            if not self.fixed == b"\x03\x00\x00\x00\x02\x00\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x03\x00\x00\x00\x02\x00\x00\x00", self.fixed, self._io, u"/types/rc4encrypted/seq/0")
            self.length = self._io.read_u4le()
            self.padding = self._io.read_bytes(8)
            self._raw_binarysd = self._io.read_bytes(self.length)
            _io__raw_binarysd = KaitaiStream(BytesIO(self._raw_binarysd))
            self.binarysd = WindefResourceData.Binarysd(_io__raw_binarysd, self, self._root)
            self.unknown1 = self._io.read_bytes(8)
            self.len_malfile = self._io.read_u8le()
            self.unknown2 = self._io.read_bytes(4)
            self.mal_file = self._io.read_bytes(self.len_malfile)


    class Binarysd(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.revision = self._io.read_u1()
            self.reserved = self._io.read_bytes(1)
            if not self.reserved == b"\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00", self.reserved, self._io, u"/types/binarysd/seq/1")
            self.control_flags = self._io.read_u2le()
            self.owner_offset = self._io.read_u4le()
            self.group_offset = self._io.read_u4le()
            self.sacl_offset = self._io.read_u4le()
            self.dacl_offset = self._io.read_u4le()

        @property
        def owner(self):
            if hasattr(self, '_m_owner'):
                return self._m_owner if hasattr(self, '_m_owner') else None

            if self.owner_offset > 0:
                _pos = self._io.pos()
                self._io.seek(self.owner_offset)
                self._m_owner = WindefResourceData.Sid(self._io, self, self._root)
                self._io.seek(_pos)

            return self._m_owner if hasattr(self, '_m_owner') else None

        @property
        def group(self):
            if hasattr(self, '_m_group'):
                return self._m_group if hasattr(self, '_m_group') else None

            if self.group_offset > 0:
                _pos = self._io.pos()
                self._io.seek(self.group_offset)
                self._m_group = WindefResourceData.Sid(self._io, self, self._root)
                self._io.seek(_pos)

            return self._m_group if hasattr(self, '_m_group') else None

        @property
        def dacl(self):
            if hasattr(self, '_m_dacl'):
                return self._m_dacl if hasattr(self, '_m_dacl') else None

            if self.dacl_offset > 0:
                _pos = self._io.pos()
                self._io.seek(self.dacl_offset)
                self._m_dacl = WindefResourceData.Acl(self._io, self, self._root)
                self._io.seek(_pos)

            return self._m_dacl if hasattr(self, '_m_dacl') else None

        @property
        def sacl(self):
            if hasattr(self, '_m_sacl'):
                return self._m_sacl if hasattr(self, '_m_sacl') else None

            if self.sacl_offset > 0:
                _pos = self._io.pos()
                self._io.seek(self.sacl_offset)
                self._m_sacl = WindefResourceData.Acl(self._io, self, self._root)
                self._io.seek(_pos)

            return self._m_sacl if hasattr(self, '_m_sacl') else None


    class Ace(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.accessallowtype = self._io.read_u1()
            self.flags = self._io.read_u1()
            self.acesize = self._io.read_u2le()
            self.accessmask = self._io.read_u4le()
            self._raw_sid = self._io.read_bytes((self.acesize - 8))
            _io__raw_sid = KaitaiStream(BytesIO(self._raw_sid))
            self.sid = WindefResourceData.Sid(_io__raw_sid, self, self._root)


    class Acl(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.revision = self._io.read_u1()
            self.reserved = self._io.read_bytes(1)
            if not self.reserved == b"\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00", self.reserved, self._io, u"/types/acl/seq/1")
            self.aclsize = self._io.read_u2le()
            self.acecount = self._io.read_u2le()
            self.reserved2 = self._io.read_bytes(2)
            if not self.reserved2 == b"\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00\x00", self.reserved2, self._io, u"/types/acl/seq/4")
            self._raw_acelist = self._io.read_bytes((self.aclsize - 8))
            _io__raw_acelist = KaitaiStream(BytesIO(self._raw_acelist))
            self.acelist = WindefResourceData.Acelist(_io__raw_acelist, self, self._root)


    class Sid(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.revision = self._io.read_u1()
            self.number_of_chunks = self._io.read_u1()
            self.reserved = self._io.read_bytes(2)
            if not self.reserved == b"\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00\x00", self.reserved, self._io, u"/types/sid/seq/2")
            self.firstchunk = self._io.read_u4be()
            self.chunk = [None] * (self.number_of_chunks)
            for i in range(self.number_of_chunks):
                self.chunk[i] = self._io.read_u4le()



    class Acelist(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ace = []
            i = 0
            while not self._io.is_eof():
                self.ace.append(WindefResourceData.Ace(self._io, self, self._root))
                i += 1







#!/usr/bin/env python3

import argparse
import csv
import tarfile
import io
import os
import sys
import ctypes
import pathlib

from colorama import Fore, Style, init
from maldump.av_manager import AVManager


__version__ = '0.1.0'


def main():

    init()
    args = parse_cli()

    # Admin privileges are required for optimal function (windows only)
    if sys.platform == 'win32' and not ctypes.windll.shell32.IsUserAnAdmin():
        print('Please try again with admin privileges')
        exit(1)

    # Save the current working directory
    cwd = os.getcwd()

    # Switch to root partition
    os.chdir(args.root_dir)

    # Get a list of all installed avs
    avs = AVManager.detect()

    if args.quar:
        export_files(avs, cwd)
    elif args.meta:
        export_meta(avs, cwd)
    elif args.all:
        export_files(avs, cwd)
        export_meta(avs, cwd)
    else:
        list_files(avs)


def export_files(avs, cwd, out_file='quarantine.tar'):
    total = 0
    for av in avs:
        entries = av.export()
        if (len(entries)) > 0:
            tar = tarfile.open(cwd + '/' + out_file, total and 'a' or 'w')
            total += len(entries)
            for entry in entries:
                tarinfo = tarfile.TarInfo(av.name + '/' + entry.md5)
                tarinfo.size = len(entry.malfile)
                tar.addfile(tarinfo, io.BytesIO(entry.malfile))
            tar.close()
    if total > 0:
        print(f"Exported {total} object(s) into '{out_file}'")


def export_meta(avs, cwd, meta_file='quarantine.csv'):
    entries = []
    for av in avs:
        for e in av.export():
            d = vars(e)
            d.update(antivirus=av.name)
            entries.append(d)
    if len(entries) > 0:
        csv_name = cwd + '/' + meta_file
        with open(csv_name, 'w', encoding='utf-8', newline='') as f:
            fields = ['timestamp', 'antivirus', 'threat', 'path', 'size', 'md5']
            writer = csv.DictWriter(f, fields, extrasaction='ignore')
            writer.writeheader()
            writer.writerows(entries)
        print(f"Written {len(entries)} row(s) into file '{meta_file}'")


def list_files(avs):
    for i, av in enumerate(avs):
        entries = av.export()
        if len(entries) > 0:
            if i != 0:
                print()
            print(Fore.YELLOW + '---', av.name, '---' + Style.RESET_ALL)
            for e in entries:
                print(e.path)


def parse_cli():
    parser = argparse.ArgumentParser(
        prog='maldump',
        formatter_class=argparse.RawTextHelpFormatter,
        description='Multi-quarantine extractor',
        epilog='Supported quarantines:\n' + '\n'.join(sorted(['  * ' + av.name for av in AVManager.avs]))
    )

    parser.add_argument(
        'root_dir', type=pathlib.Path,
        help=r'root directory where OS is installed (example C:\)'
    )
    parser.add_argument(
        '-l', '--list', action='store_true',
        help='list quarantined file(s) to stdout (default action)'
    )
    parser.add_argument(
        '-q', '--quar', action='store_true',
        help='dump quarantined file(s) to archive \'quarantine.tar\''
    )
    parser.add_argument(
        '-m', '--meta', action='store_true',
        help='dump metadata to CSV file \'quarantine.csv\''
    )
    parser.add_argument(
        '-a', '--all', action='store_true',
        help='equivalent of running both -q and -m'
    )
    parser.add_argument(
        '-v', '--version', action='version', version='%(prog)s ' + __version__
    )

    return parser.parse_args()


if __name__ == "__main__":
    main()



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import maldump.utils as utils
from enum import Enum


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class WindefEntries(KaitaiStruct):
    """Creator: Florian Bausch, ERNW Research GmbH, https://ernw-research.de
    License: CC-BY-SA-4.0 https://creativecommons.org/licenses/by-sa/4.0/
    Modified-by: Nikola Kne�evi�
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self._raw__raw_header = self._io.read_bytes(60)
        _process = utils.CustomArc4(b"\x1E\x87\x78\x1B\x8D\xBA\xA8\x44\xCE\x69\x70\x2C\x0C\x78\xB7\x86\xA3\xF6\x23\xB7\x38\xF5\xED\xF9\xAF\x83\x53\x0F\xB3\xFC\x54\xFA\xA2\x1E\xB9\xCF\x13\x31\xFD\x0F\x0D\xA9\x54\xF6\x87\xCB\x9E\x18\x27\x96\x97\x90\x0E\x53\xFB\x31\x7C\x9C\xBC\xE4\x8E\x23\xD0\x53\x71\xEC\xC1\x59\x51\xB8\xF3\x64\x9D\x7C\xA3\x3E\xD6\x8D\xC9\x04\x7E\x82\xC9\xBA\xAD\x97\x99\xD0\xD4\x58\xCB\x84\x7C\xA9\xFF\xBE\x3C\x8A\x77\x52\x33\x55\x7D\xDE\x13\xA8\xB1\x40\x87\xCC\x1B\xC8\xF1\x0F\x6E\xCD\xD0\x83\xA9\x59\xCF\xF8\x4A\x9D\x1D\x50\x75\x5E\x3E\x19\x18\x18\xAF\x23\xE2\x29\x35\x58\x76\x6D\x2C\x07\xE2\x57\x12\xB2\xCA\x0B\x53\x5E\xD8\xF6\xC5\x6C\xE7\x3D\x24\xBD\xD0\x29\x17\x71\x86\x1A\x54\xB4\xC2\x85\xA9\xA3\xDB\x7A\xCA\x6D\x22\x4A\xEA\xCD\x62\x1D\xB9\xF2\xA2\x2E\xD1\xE9\xE1\x1D\x75\xBE\xD7\xDC\x0E\xCB\x0A\x8E\x68\xA2\xFF\x12\x63\x40\x8D\xC8\x08\xDF\xFD\x16\x4B\x11\x67\x74\xCD\x0B\x9B\x8D\x05\x41\x1E\xD6\x26\x2E\x42\x9B\xA4\x95\x67\x6B\x83\x98\xDB\x2F\x35\xD3\xC1\xB9\xCE\xD5\x26\x36\xF2\x76\x5E\x1A\x95\xCB\x7C\xA4\xC3\xDD\xAB\xDD\xBF\xF3\x82\x53")
        self._raw_header = _process.decode(self._raw__raw_header)
        _io__raw_header = KaitaiStream(BytesIO(self._raw_header))
        self.header = WindefEntries.Rc4encryptedHeader(_io__raw_header, self, self._root)
        self._raw__raw_data1 = self._io.read_bytes(self.header.len1)
        _process = utils.CustomArc4(b"\x1E\x87\x78\x1B\x8D\xBA\xA8\x44\xCE\x69\x70\x2C\x0C\x78\xB7\x86\xA3\xF6\x23\xB7\x38\xF5\xED\xF9\xAF\x83\x53\x0F\xB3\xFC\x54\xFA\xA2\x1E\xB9\xCF\x13\x31\xFD\x0F\x0D\xA9\x54\xF6\x87\xCB\x9E\x18\x27\x96\x97\x90\x0E\x53\xFB\x31\x7C\x9C\xBC\xE4\x8E\x23\xD0\x53\x71\xEC\xC1\x59\x51\xB8\xF3\x64\x9D\x7C\xA3\x3E\xD6\x8D\xC9\x04\x7E\x82\xC9\xBA\xAD\x97\x99\xD0\xD4\x58\xCB\x84\x7C\xA9\xFF\xBE\x3C\x8A\x77\x52\x33\x55\x7D\xDE\x13\xA8\xB1\x40\x87\xCC\x1B\xC8\xF1\x0F\x6E\xCD\xD0\x83\xA9\x59\xCF\xF8\x4A\x9D\x1D\x50\x75\x5E\x3E\x19\x18\x18\xAF\x23\xE2\x29\x35\x58\x76\x6D\x2C\x07\xE2\x57\x12\xB2\xCA\x0B\x53\x5E\xD8\xF6\xC5\x6C\xE7\x3D\x24\xBD\xD0\x29\x17\x71\x86\x1A\x54\xB4\xC2\x85\xA9\xA3\xDB\x7A\xCA\x6D\x22\x4A\xEA\xCD\x62\x1D\xB9\xF2\xA2\x2E\xD1\xE9\xE1\x1D\x75\xBE\xD7\xDC\x0E\xCB\x0A\x8E\x68\xA2\xFF\x12\x63\x40\x8D\xC8\x08\xDF\xFD\x16\x4B\x11\x67\x74\xCD\x0B\x9B\x8D\x05\x41\x1E\xD6\x26\x2E\x42\x9B\xA4\x95\x67\x6B\x83\x98\xDB\x2F\x35\xD3\xC1\xB9\xCE\xD5\x26\x36\xF2\x76\x5E\x1A\x95\xCB\x7C\xA4\xC3\xDD\xAB\xDD\xBF\xF3\x82\x53")
        self._raw_data1 = _process.decode(self._raw__raw_data1)
        _io__raw_data1 = KaitaiStream(BytesIO(self._raw_data1))
        self.data1 = WindefEntries.EncryptedData1(_io__raw_data1, self, self._root)
        self._raw__raw_data2 = self._io.read_bytes(self.header.len2)
        _process = utils.CustomArc4(b"\x1E\x87\x78\x1B\x8D\xBA\xA8\x44\xCE\x69\x70\x2C\x0C\x78\xB7\x86\xA3\xF6\x23\xB7\x38\xF5\xED\xF9\xAF\x83\x53\x0F\xB3\xFC\x54\xFA\xA2\x1E\xB9\xCF\x13\x31\xFD\x0F\x0D\xA9\x54\xF6\x87\xCB\x9E\x18\x27\x96\x97\x90\x0E\x53\xFB\x31\x7C\x9C\xBC\xE4\x8E\x23\xD0\x53\x71\xEC\xC1\x59\x51\xB8\xF3\x64\x9D\x7C\xA3\x3E\xD6\x8D\xC9\x04\x7E\x82\xC9\xBA\xAD\x97\x99\xD0\xD4\x58\xCB\x84\x7C\xA9\xFF\xBE\x3C\x8A\x77\x52\x33\x55\x7D\xDE\x13\xA8\xB1\x40\x87\xCC\x1B\xC8\xF1\x0F\x6E\xCD\xD0\x83\xA9\x59\xCF\xF8\x4A\x9D\x1D\x50\x75\x5E\x3E\x19\x18\x18\xAF\x23\xE2\x29\x35\x58\x76\x6D\x2C\x07\xE2\x57\x12\xB2\xCA\x0B\x53\x5E\xD8\xF6\xC5\x6C\xE7\x3D\x24\xBD\xD0\x29\x17\x71\x86\x1A\x54\xB4\xC2\x85\xA9\xA3\xDB\x7A\xCA\x6D\x22\x4A\xEA\xCD\x62\x1D\xB9\xF2\xA2\x2E\xD1\xE9\xE1\x1D\x75\xBE\xD7\xDC\x0E\xCB\x0A\x8E\x68\xA2\xFF\x12\x63\x40\x8D\xC8\x08\xDF\xFD\x16\x4B\x11\x67\x74\xCD\x0B\x9B\x8D\x05\x41\x1E\xD6\x26\x2E\x42\x9B\xA4\x95\x67\x6B\x83\x98\xDB\x2F\x35\xD3\xC1\xB9\xCE\xD5\x26\x36\xF2\x76\x5E\x1A\x95\xCB\x7C\xA4\xC3\xDD\xAB\xDD\xBF\xF3\x82\x53")
        self._raw_data2 = _process.decode(self._raw__raw_data2)
        _io__raw_data2 = KaitaiStream(BytesIO(self._raw_data2))
        self.data2 = WindefEntries.EncryptedData2(_io__raw_data2, self, self._root)

    class EncryptedData2(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.number_of_entries = self._io.read_u4le()
            self.offset = [None] * (self.number_of_entries)
            for i in range(self.number_of_entries):
                self.offset[i] = self._io.read_u4le()


        class Entrys(KaitaiStruct):
            def __init__(self, i, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.i = i
                self._read()

            def _read(self):
                self._raw_entry = self._io.read_bytes((self._parent.offset[(self.i + 1)] - self._parent.offset[self.i]))
                _io__raw_entry = KaitaiStream(BytesIO(self._raw_entry))
                self.entry = WindefEntries.Entry(_io__raw_entry, self, self._root)


        class Entrye(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.entry = WindefEntries.Entry(self._io, self, self._root)


        @property
        def entries(self):
            if hasattr(self, '_m_entries'):
                return self._m_entries if hasattr(self, '_m_entries') else None

            _pos = self._io.pos()
            self._io.seek(self.offset[0])
            self._m_entries = [None] * (self.number_of_entries)
            for i in range(self.number_of_entries):
                _on = i != (self.number_of_entries - 1)
                if _on == True:
                    self._m_entries[i] = WindefEntries.EncryptedData2.Entrys(i, self._io, self, self._root)
                elif _on == False:
                    self._m_entries[i] = WindefEntries.EncryptedData2.Entrye(self._io, self, self._root)

            self._io.seek(_pos)
            return self._m_entries if hasattr(self, '_m_entries') else None


    class Guid(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.id1 = self._io.read_u4le()
            self.id2 = self._io.read_u2le()
            self.id3 = self._io.read_u2le()
            self.id4 = self._io.read_bytes(2)
            self.id5 = self._io.read_bytes(6)


    class Entry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.path = WindefEntries.NullTerminatedUtf16le(self._io, self, self._root)
            self.number_of_elements = self._io.read_u2le()
            self.typestr = (self._io.read_bytes_term(0, False, True, True)).decode(u"UTF8")
            self.padding = self._io.read_bytes(((4 - self._io.pos()) % 4))
            self.element = [None] * (self.number_of_elements)
            for i in range(self.number_of_elements):
                self.element[i] = WindefEntries.Listelement(self._io, self, self._root)



    class Winfiletime(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ts = self._io.read_u8le()

        @property
        def unixts(self):
            if hasattr(self, '_m_unixts'):
                return self._m_unixts if hasattr(self, '_m_unixts') else None

            self._m_unixts = ((self.ts * 1E-7) - 11644473600)
            return self._m_unixts if hasattr(self, '_m_unixts') else None


    class EncryptedData1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.guid = WindefEntries.Guid(self._io, self, self._root)
            self.unknown3 = self._io.read_bytes(16)
            self.time = WindefEntries.Winfiletime(self._io, self, self._root)
            self.id11 = self._io.read_u4le()
            self.id12 = self._io.read_u2le()
            self.id13 = self._io.read_u2le()
            self.number_of_strings = self._io.read_u4le()
            self.mal_type = (self._io.read_bytes_term(0, False, True, True)).decode(u"UTF-8")


    class StrUtf16le(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.value = (self._io.read_bytes_full()).decode(u"UTF-16LE")


    class NullTerminatedUtf16le(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.character = []
            i = 0
            while True:
                _ = self._io.read_u2le()
                self.character.append(_)
                if self.character[i] == 0:
                    break
                i += 1


    class Hash(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.value = self._io.read_bytes(20)


    class Listelement(KaitaiStruct):

        class Elementtypes(Enum):
            utf16 = 32
            uint4 = 48
            hash = 64
            winfiletime = 96
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.length = self._io.read_u2le()
            self.elementsubtype = self._io.read_u1()
            self.elementtype = KaitaiStream.resolve_enum(WindefEntries.Listelement.Elementtypes, self._io.read_u1())
            _on = self.elementtype
            if _on == WindefEntries.Listelement.Elementtypes.winfiletime:
                self._raw_content = self._io.read_bytes(self.length)
                _io__raw_content = KaitaiStream(BytesIO(self._raw_content))
                self.content = WindefEntries.Winfiletime(_io__raw_content, self, self._root)
            elif _on == WindefEntries.Listelement.Elementtypes.utf16:
                self._raw_content = self._io.read_bytes(self.length)
                _io__raw_content = KaitaiStream(BytesIO(self._raw_content))
                self.content = WindefEntries.StrUtf16le(_io__raw_content, self, self._root)
            elif _on == WindefEntries.Listelement.Elementtypes.hash:
                self._raw_content = self._io.read_bytes(self.length)
                _io__raw_content = KaitaiStream(BytesIO(self._raw_content))
                self.content = WindefEntries.Hash(_io__raw_content, self, self._root)
            elif _on == WindefEntries.Listelement.Elementtypes.uint4:
                self.content = self._io.read_u4le()
            else:
                self.content = self._io.read_bytes(self.length)
            self.padding = self._io.read_bytes(((4 - self._io.pos()) % 4))


    class Rc4encryptedHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.magic = self._io.read_bytes(16)
            if not self.magic == b"\xDB\xE8\xC5\x01\x01\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\xDB\xE8\xC5\x01\x01\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00", self.magic, self._io, u"/types/rc4encrypted_header/seq/0")
            self.unknown1 = self._io.read_bytes(24)
            self.len1 = self._io.read_u4le()
            self.len2 = self._io.read_u4le()
            self.unknown2 = self._io.read_bytes(12)






from maldump.avs import *


class AVManager():
    """Container class holding all instances"""
    avs = [
        windef.WindowsDefender(),
        forticlient.FortiClient(),
        malwarebytes.Malwarebytes(),
        gdata.GData(),
        avast.Avast(),
        avira.Avira(),
        kaspersky.Kaspersky(),
    ]

    @classmethod
    def detect(cls):
        """Returns a list of avs installed on the system"""
        return [av for av in cls.avs if av.location.exists()]



import sqlite3

from .quarantine import *
from maldump.utils import xor


class Kaspersky(Quarantine):
    """Implements Kaspersky quarantine format"""

    def __init__(self):
        super().__init__()
        self.name = 'Kaspersky for Windows Server'
        self.location = Path('ProgramData/Kaspersky Lab/Kaspersky Security for Windows Server/11.0/Quarantine')

    def _normalize_time(self, number):
        year = (number >> 48) & 0xFFFF
        month = (number >> 40) & 0xFF
        days = (number >> 32) & 0xFF
        hours = (number >> 24) & 0xFF
        minutes = (number >> 16) & 0xFF
        seconds = (number >> 8) & 0xFF

        return dt(year, month, days, hours, minutes, seconds)

    def _get_malfile(self, data):
        file = self.location / data
        key = [0xE2, 0x45, 0x48, 0xEC, 0x69, 0x0E, 0x5C, 0xAC]
        with open(file, 'rb') as f:
            return xor(f.read(), key)

    def export(self):
        try:
            conn = sqlite3.connect(str(self.location / 'quarantine.db'))
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM 'objects'")
            rows = cursor.fetchall()
        except Exception as e:
            print('Kaspersky DB Error: ' + str(e))

        quarfiles = []
        for row in rows:
            malfile = self._get_malfile(row[0])
            q = QuarEntry()
            q.timestamp = self._normalize_time(row[6])
            q.threat = row[3]
            q.path = row[1] + row[2]
            q.size = row[7]
            q.md5 = md5(malfile).digest().hex()
            q.malfile = malfile
            quarfiles.append(q)

        conn.close()
        return quarfiles
